package TwoFactorAuthentication::CMS;
use strict;
use warnings;
use utf8;
use TwoFactorAuthentication::Util qw( plugin is_mt70_or_later );
use TwoFactorAuthentication::Auth;

use MT::Util qw( encode_url );

use constant TIME_STEP => 30;

sub _errtrans {
    my $app = shift;
    return $app->error(plugin->translate(@_));
}

sub verify {
    my $app = shift;

    return $app->return_to_user_dashboard()
        if $app->config('DisableTwoFactorAuthentication');

    my $sess = $app->{session}
        or $app->logout();
    if($sess && $sess->get('tfa_verified')) {
        return $app->return_to_user_dashboard();
    }

    my $params = {};
    $params->{query_params} = [
        { name => '__mode', value => 'tfa_do_verify' },
    ];
    if($app->errstr) {
        $params->{error} = $app->errstr;
    }
    my $tmpl_file = is_mt70_or_later()
        ? 'verify_v7.tmpl'
        : 'verify.tmpl';
    return plugin->load_tmpl($tmpl_file, $params);
}

sub do_verify {
    my $app = shift;

    return $app->return_to_user_dashboard()
        if $app->config('DisableTwoFactorAuthentication');

    my $code = $app->param('code');
    unless($code) {
        _errtrans($app, 'Enter verification code.');
        return verify($app);
    }

    my $user = $app->user
        or return _errtrans($app, 'No user.');

    my $auth = TwoFactorAuthentication::Auth->new(user => $user, app => $app);
    unless($auth->is_valid_totp($code)) {
        _errtrans($app, $auth->errstr);
        return verify($app);
    }

    my $sess = $app->{session};
    if($sess) {
        my $query_params = $sess->get('tfa_query_params');
        if($query_params) {
            my $mode;
            foreach my $pair (@$query_params) {
                $app->param($pair->{name}, $pair->{value});
                if ($pair->{name} eq '__mode') {
                    $mode = $pair->{value};
                }
            }
            $sess->set('tfa_query_params', undef);
            $sess->save;
            return $app->forward($mode) if $mode;
        }
    }

    return $app->return_to_user_dashboard();
}

sub show_qrcode {
    my $app = shift;

    my $author_id = $app->param('id');
    my $user = $app->model('author')->load($author_id)
        or return _errtrans($app, 'No author.');

    my $auth = TwoFactorAuthentication::Auth->new(user => $user, app => $app);

    my $params = {
        qrcode_src => $auth->qrcode_src,
        qrcode_url => $auth->qrcode_url,
        tfa_secret_key => $auth->secret_key,
    };
    my $tmpl_file = is_mt70_or_later()
        ? 'dialog/tfa_enable_v7.tmpl'
        : 'dialog/tfa_enable.tmpl';
    return $app->load_tmpl($tmpl_file, $params);
}

sub enable {
    my $app = shift;
    my (%params) = @_;

    my $author_id = $app->param('id');
    my $current_user = $app->user
        or return;
    return $app->permission_denied
        unless $author_id == $current_user->id;

    my $user = $app->model('author')->load($author_id)
        or return _errtrans($app, 'No author.');

    my $auth = TwoFactorAuthentication::Auth->new(
        user => $user,
        app => $app,
        secret => $params{secret},
    );
    $auth->enabled()
        and return _errtrans($app, 'Already enabled.');

    my $params = {
        %params,
        id => $author_id,
        qrcode_src => $auth->qrcode_src,
        qrcode_url => $auth->qrcode_url,
        tfa_secret_key => $auth->secret_key,
    };
    my $tmpl_file = is_mt70_or_later()
        ? 'dialog/tfa_enable_v7.tmpl'
        : 'dialog/tfa_enable.tmpl';
    return $app->load_tmpl($tmpl_file, $params);
}

sub do_enable {
    my $app = shift;

    my $author_id = $app->param('id');
    my $current_user = $app->user
        or return;
    return $app->permission_denied
        unless $author_id == $current_user->id;

    my $user = $app->model('author')->load($author_id)
        or return _errtrans($app, 'No author.');

    my $auth = TwoFactorAuthentication::Auth->new(
        user => $user,
        app => $app,
    );

    $auth->enabled()
        and return _errtrans($app, 'Already enabled.');

    my $secret = $app->param('tfa_secret_key')
        or return _errtrans($app, 'No secret key.');
    unless($auth->secret_key eq $secret) {
        return _errtrans($app, 'Invalid secret key.');
    }

    my $code = $app->param('code')
        or return enable(
            $app,
            message => plugin->translate('Enter verification code.'),
            secret => $secret
        );

    unless($auth->is_valid_totp($code)) {
        return enable(
            $app,
            message => plugin->translate('Invalid verification code.'),
            secret => $secret
        );
    }

    $auth->enable()
        or return _errtrans($app, $auth->errstr);

    my $sess = $app->{session}
        or return _errtrans($app, 'Failed to get session.');
    $sess->set('tfa_verified', 1);
    $sess->save;

    # my $return_url = $app->base . $app->uri(
    my $return_url = $app->uri(
        mode => 'view',
        args => {
            '_type' => 'author',
            id => $author_id,
        }
    );

    my $params = {
        id => $author_id,
        message => plugin->translate('Two factor authentication enabled.'),
        recovery_codes => $auth->recovery_codes,
        return_url => $return_url,
    };

    my $tmpl_file = is_mt70_or_later()
        ? 'dialog/tfa_changed_v7.tmpl'
        : 'dialog/tfa_changed.tmpl';
    return $app->load_tmpl($tmpl_file, $params);
}

sub disable {
    my $app = shift;

    my $author_id = $app->param('id');
    my $user = $app->model('author')->load($author_id)
        or return _errtrans($app, 'No author.');

    my $auth = TwoFactorAuthentication::Auth->new(
        user => $user,
        app => $app,
    );
    $auth->enabled
        or return _errtrans($app, 'Already disabled.');
    $auth->disable
        or return _errtrans($app, $auth->errstr);

    my $sess = $app->{session}
        or return _errtrans($app, 'Failed to get session.');
    $sess->set('tfa_verified', 0);
    $sess->save;

    # my $return_url = $app->base . $app->uri(
    my $return_url = $app->uri(
        mode => 'view',
        args => {
            '_type' => 'author',
            id => $author_id,
        }
    );

    my $params = {
        id => $author_id,
        message => plugin->translate('Two factor authentication disabled.'),
        return_url => $return_url,
        disabled => 1,
    };

    my $tmpl_file = is_mt70_or_later()
        ? 'dialog/tfa_changed_v7.tmpl'
        : 'dialog/tfa_changed.tmpl';
    return $app->load_tmpl($tmpl_file, $params);
}

sub view_recovery_codes {
    my $app = shift;

    my $author_id = $app->param('id');

    my $current_user = $app->user
        or return;
    return $app->permission_denied
        unless $author_id == $current_user->id;

    my $user = $app->model('author')->load($author_id)
        or return _errtrans($app, 'No author.');

    my $auth = TwoFactorAuthentication::Auth->new(
        user => $user,
        app => $app,
    );

    my $params = {
        id => $author_id,
        recovery_codes => $auth->recovery_codes,
    };

    my $tmpl_file = is_mt70_or_later()
        ? 'dialog/tfa_changed_v7.tmpl'
        : 'dialog/tfa_changed.tmpl';
    return $app->load_tmpl($tmpl_file, $params);
}

sub generate_recovery_codes {
    my $app = shift;

    my $author_id = $app->param('id');
    my $current_user = $app->user
        or return;
    return $app->permission_denied
        unless $author_id == $current_user->id;

    my $user = $app->model('author')->load($author_id)
        or return _errtrans($app, 'No author.');

    my $auth = TwoFactorAuthentication::Auth->new(
        user => $user,
        app => $app,
    );
    $auth->regenerate_recovery_codes
        or return _errtrans($app, $auth->errstr);

    my $params = {
        id => $author_id,
        message => plugin->translate('New recovery codes generated.'),
        recovery_codes => $auth->recovery_codes,
    };

    my $tmpl_file = is_mt70_or_later()
        ? 'dialog/tfa_changed_v7.tmpl'
        : 'dialog/tfa_changed.tmpl';
    return $app->load_tmpl($tmpl_file, $params);
}

1;
__END__
